# Generated by Django 5.0.6 on 2024-08-05 00:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("slurpit_netbox", "0012_slurpitprefix"),
    ]

    operations = [
        migrations.AddField(
            model_name="slurpitinitipaddress",
            name="ignore_status",
            field=models.BooleanField(default=False),
        ),
    ]
