# Generated by Django 5.0.6 on 2024-08-05 01:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("slurpit_netbox", "0014_slurpitinitipaddress_ignore_description_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="slurpitinterface",
            name="ignore_duplex",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="slurpitinterface",
            name="ignore_module",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="slurpitinterface",
            name="ignore_speed",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="slurpitinterface",
            name="ignore_type",
            field=models.BooleanField(default=False),
        ),
    ]
