# Generated by Django 4.2.9 on 2024-04-18 07:40

from django.db import migrations, models
import django.db.models.deletion
import taggit.managers
import utilities.json


class Migration(migrations.Migration):

    dependencies = [
        ('extras', '0105_customfield_min_max_values'),
        ('tenancy', '0014_contactassignment_ordering'),
        ('ipam', '0069_gfk_indexes'),
        ('slurpit_netbox', '0006_slurpitsnapshot_result_type'),
    ]

    operations = [
        migrations.CreateModel(
            name='SlurpitInitIPAddress',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('description', models.CharField(blank=True, max_length=200)),
                ('comments', models.TextField(blank=True)),
                ('status', models.CharField(default='active', max_length=50)),
                ('role', models.CharField(blank=True, max_length=50)),
                ('enable_reconcile', models.BooleanField(default=False)),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
                ('tenant', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='slurpit_ip_addresses', to='tenancy.tenant')),
                ('vrf', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='slurpit_ip_addresses', to='ipam.vrf')),
            ],
            options={
                'verbose_name': 'Slurpit Init IP address',
                'verbose_name_plural': 'Slurpit Init IP addresses',
            },
        ),
    ]
