# Generated by Django 5.0.9 on 2024-10-30 09:10

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("slurpit_netbox", "0021_slurpitimporteddevice_site_slurpitstageddevice_site"),
    ]

    operations = [
        migrations.DeleteModel(
            name="SlurpitLog",
        ),
    ]
