# Generated by Django 4.2.9 on 2024-05-01 02:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('slurpit_netbox', '0009_slurpitinterface'),
    ]

    operations = [
        migrations.AlterField(
            model_name='slurpitinterface',
            name='name',
            field=models.CharField(blank=True, max_length=64),
        ),
    ]
