# Generated by Django 5.1.5 on 2025-01-20 11:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('slurpit_netbox', '0022_delete_slurpitlog'),
    ]

    operations = [
        migrations.AlterField(
            model_name='slurpitinterface',
            name='cable_end',
            field=models.CharField(blank=True, max_length=1, null=True),
        ),
    ]
