# Generated by Django 4.2.9 on 2024-05-07 06:16

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('dcim', '0185_gfk_indexes'),
        ('slurpit_netbox', '0010_alter_slurpitinterface_name'),
    ]

    operations = [
        migrations.AlterField(
            model_name='slurpitinterface',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='%(class)ss', to='dcim.device'),
        ),
    ]
